@@ -3,7 +3,6 @@
 
 # Copyright (c) 2021, 2022 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
-from copy import deepcopy
 from typing import List, Union
 from urllib.parse import urlparse
 
@@ -11,14 +10,16 @@ import fsspec
 from ads.common.auth import default_signer
 from ads.jobs.builders.base import Builder
 from ads.jobs.builders.infrastructure.dataflow import DataFlow, DataFlowRun
-from ads.jobs.builders.infrastructure.dsc_job import (DataScienceJob,
-                                                      DataScienceJobRun)
+from ads.jobs.builders.infrastructure.dsc_job import DataScienceJob, DataScienceJobRun
 from ads.jobs.builders.runtimes.container_runtime import ContainerRuntime
-from ads.jobs.builders.runtimes.python_runtime import (DataFlowRuntime,
-                                                       GitPythonRuntime,
-                                                       NotebookRuntime,
-                                                       PythonRuntime, Runtime,
-                                                       ScriptRuntime)
+from ads.jobs.builders.runtimes.python_runtime import (
+    DataFlowRuntime,
+    GitPythonRuntime,
+    NotebookRuntime,
+    PythonRuntime,
+    Runtime,
+    ScriptRuntime,
+)
 
 
 class Job(Builder):
@@ -39,7 +40,8 @@ class Job(Builder):
                 .with_compartment_id("<compartment_ocid>")
                 .with_project_id("<project_ocid>")
                 .with_subnet_id("<subnet_ocid>")
-                .with_shape_name("VM.Standard2.1")
+                .with_shape_name("VM.Standard.E3.Flex")
+                .with_shape_config_details(memory_in_gbs=16, ocpus=1)
                 .with_block_storage_size(50)
                 .with_log_group_id("<log_group_ocid>")
                 .with_log_id("<log_ocid>")
@@ -211,6 +213,8 @@ class Job(Builder):
             If it is None, a default name may be generated by the infrastructure,
             depending on the implementation of the infrastructure.
             For OCI data science job, the default name contains the job artifact name and a timestamp.
+            If no artifact, a randomly generated easy to remember name with timestamp will be generated,
+            like 'strange-spider-2022-08-17-23:55.02'.
         infrastructure : Infrastructure, optional
             Job infrastructure, by default None
         runtime : Runtime, optional
@@ -354,6 +358,8 @@ class Job(Builder):
             The infrastructure handles the naming of the job run.
             For data science job, if a name is not provided,
             a default name will be generated containing the job name and the timestamp of the run.
+            If no artifact, a randomly generated easy to remember name
+            with timestamp will be generated, like 'strange-spider-2022-08-17-23:55.02'.
         args : str, optional
             Command line arguments for the job run, by default None.
             This will override the configurations on the job.
