@@ -9,6 +9,9 @@ from ads.jobs.serializer import Serializable
 
 
 class Builder(Serializable):
+
+    attribute_map = {}
+
     def __init__(self, spec: Dict = None, **kwargs) -> None:
         """Initialize the object with specifications.
 
@@ -23,10 +26,30 @@ class Builder(Serializable):
             If spec contains the same key as the one in kwargs, the value from kwargs will be used.
         """
         super().__init__()
-        if spec is None:
-            spec = {}
-        spec.update(kwargs)
-        self._spec = spec
+        self._spec = self._load_default_properties()
+        self._spec.update(self._standardize_spec(spec))
+        self._spec.update(self._standardize_spec(kwargs))
+
+    def _load_default_properties(self):
+        """
+        Load default properties from environment variables, notebook session, etc.
+        Should be implemented in the child classes.
+
+        Returns
+        -------
+        Dict
+            A dictionary of default properties.
+        """
+        return {}
+
+    def _standardize_spec(self, spec):
+        if not spec:
+            return {}
+        snake_to_camel_map = {v: k for k, v in self.attribute_map.items()}
+        for key in list(spec.keys()):
+            if key not in self.attribute_map and key in snake_to_camel_map:
+                spec[snake_to_camel_map[key]] = spec.pop(key)
+        return spec
 
     def set_spec(self, k: str, v: Any):
         """Sets a specification property for the object.
