@@ -7,7 +7,7 @@ from __future__ import annotations
 
 import json
 import os
-from typing import Any, Dict
+from typing import Any
 
 from ads.jobs.builders.runtimes.base import Runtime
 from ads.common.auth import default_signer
@@ -25,6 +25,9 @@ class CondaRuntime(Runtime):
     CONST_CONDA_URI = "uri"
     CONST_CONDA_REGION = "region"
 
+    attribute_map = {CONST_CONDA: CONST_CONDA}
+    attribute_map.update(Runtime.attribute_map)
+
     @property
     def conda(self) -> dict:
         """The conda pack specification
@@ -100,6 +103,12 @@ class ScriptRuntime(CondaRuntime):
     CONST_ENTRYPOINT = "entrypoint"
     CONST_SCRIPT_PATH = "scriptPathURI"
 
+    attribute_map = {
+        CONST_ENTRYPOINT: CONST_ENTRYPOINT,
+        CONST_SCRIPT_PATH: "script_path_uri",
+    }
+    attribute_map.update(CondaRuntime.attribute_map)
+
     @property
     def script_uri(self) -> str:
         """The URI of the source code"""
@@ -181,6 +190,15 @@ class _PythonRuntimeMixin(Runtime):
     CONST_ENTRYPOINT = "entrypoint"
     CONST_ENTRY_FUNCTION = "entryFunction"
 
+    attribute_map = {
+        CONST_OUTPUT_DIR: "output_dir",
+        CONST_OUTPUT_URI: "output_uri",
+        CONST_PYTHON_PATH: "python_path",
+        CONST_ENTRYPOINT: CONST_ENTRYPOINT,
+        CONST_ENTRY_FUNCTION: "entry_function",
+    }
+    attribute_map.update(Runtime.attribute_map)
+
     def with_output(self, output_dir: str, output_uri: str):
         """Specifies the outputs of the job.
         The output files in output_dir will be copied to remote output_uri when the job is finished.
@@ -278,6 +296,9 @@ class PythonRuntime(ScriptRuntime, _PythonRuntimeMixin):
     """Represents a job runtime using ADS driver script to run Python code"""
 
     CONST_WORKING_DIR = "workingDir"
+    attribute_map = {CONST_WORKING_DIR: "working_dir"}
+    attribute_map.update(ScriptRuntime.attribute_map)
+    attribute_map.update(_PythonRuntimeMixin.attribute_map)
 
     def with_working_dir(self, working_dir: str):
         """Specifies the working directory in the job run.
@@ -309,7 +330,15 @@ class NotebookRuntime(CondaRuntime):
     CONST_NOTEBOOK_PATH = "notebookPathURI"
     CONST_NOTEBOOK_ENCODING = "notebookEncoding"
     CONST_OUTPUT_URI = "outputURI"
-    EXCLUDE_TAG = "excludeTags"
+    CONST_EXCLUDE_TAG = "excludeTags"
+
+    attribute_map = {
+        CONST_NOTEBOOK_PATH: "notebook_path_uri",
+        CONST_NOTEBOOK_ENCODING: "notebook_encoding",
+        CONST_OUTPUT_URI: "output_uri",
+        CONST_EXCLUDE_TAG: "exclude_tags",
+    }
+    attribute_map.update(CondaRuntime.attribute_map)
 
     @property
     def notebook_uri(self) -> str:
@@ -321,7 +350,7 @@ class NotebookRuntime(CondaRuntime):
         """The encoding of the notebook"""
         return self.get_spec(self.CONST_NOTEBOOK_ENCODING)
 
-    def with_notebook(self, path: str, encoding='utf-8'):
+    def with_notebook(self, path: str, encoding="utf-8"):
         """Specifies the notebook to be converted to python script and run as a job.
 
         Parameters
@@ -340,7 +369,7 @@ class NotebookRuntime(CondaRuntime):
     @property
     def exclude_tag(self) -> list:
         """A list of cell tags indicating cells to be excluded from the job"""
-        return self.get_spec(self.EXCLUDE_TAG, [])
+        return self.get_spec(self.CONST_EXCLUDE_TAG, [])
 
     def with_exclude_tag(self, *tags):
         """Specifies the cell tags in the notebook to exclude cells from the job script.
@@ -361,7 +390,7 @@ class NotebookRuntime(CondaRuntime):
                 exclude_tag_list.extend(tag)
             else:
                 exclude_tag_list.append(tag)
-        return self.set_spec(self.EXCLUDE_TAG, exclude_tag_list)
+        return self.set_spec(self.CONST_EXCLUDE_TAG, exclude_tag_list)
 
     @property
     def output_uri(self) -> list:
@@ -393,33 +422,29 @@ class GitPythonRuntime(CondaRuntime, _PythonRuntimeMixin):
     CONST_COMMIT = "commit"
     CONST_GIT_SSH_SECRET_ID = "gitSecretId"
     CONST_SKIP_METADATA = "skipMetadataUpdate"
-
-    def __init__(self, spec: Dict = None, skip_metadata_update=False) -> None:
-        """Initialize Git Python Runtime.
-
-        Parameters
-        ----------
-        spec : dict, optional
-            Runtime specifications, by default None
-        skip_metadata_update : bool, optional
-            Indicate if the metadata update should be skipped after the job run, by default False.
-            By default, the job run metadata will be updated with the following freeform tags:
-            * repo: The URL of the Git repository
-            * commit: The Git commit ID
-            * module: The entry script/module
-            * method: The entry function/method
-            * outputs. The prefix of the output files in object storage.
-
-            This update step also requires resource principals to have the permission to update the job run.
-        """
-        super().__init__(spec=spec)
-        if skip_metadata_update:
-            self.set_spec(self.CONST_SKIP_METADATA, True)
+    attribute_map = {
+        CONST_GIT_URL: CONST_GIT_URL,
+        CONST_BRANCH: CONST_BRANCH,
+        CONST_COMMIT: CONST_COMMIT,
+        CONST_GIT_SSH_SECRET_ID: "git_secret_id",
+        CONST_SKIP_METADATA: "skip_metadata_update",
+    }
+    attribute_map.update(CondaRuntime.attribute_map)
+    attribute_map.update(_PythonRuntimeMixin.attribute_map)
 
     @property
     def skip_metadata_update(self):
         """Indicate if the metadata update should be skipped after the job run
 
+        By default, the job run metadata will be updated with the following freeform tags:
+        * repo: The URL of the Git repository
+        * commit: The Git commit ID
+        * module: The entry script/module
+        * method: The entry function/method
+        * outputs. The prefix of the output files in object storage.
+
+        This update step also requires resource principals to have the permission to update the job run.
+
         Returns
         -------
         bool
@@ -521,12 +546,78 @@ class GitPythonRuntime(CondaRuntime, _PythonRuntimeMixin):
         return self.get_spec(self.CONST_GIT_SSH_SECRET_ID)
 
 
-class DataFlowRuntime(Runtime):
+class DataFlowRuntime(CondaRuntime):
 
     CONST_SCRIPT_BUCKET = "scriptBucket"
     CONST_ARCHIVE_BUCKET = "archiveBucket"
     CONST_ARCHIVE_URI = "archiveUri"
     CONST_SCRIPT_PATH = "scriptPathURI"
+    CONST_CONFIGURATION = "configuration"
+    CONST_CONDA_AUTH_TYPE = "condaAuthType"
+    attribute_map = {
+        CONST_SCRIPT_BUCKET: "script_bucket",
+        CONST_ARCHIVE_URI: "archive_bucket",
+        CONST_ARCHIVE_URI: "archive_uri",
+        CONST_SCRIPT_PATH: "script_path_uri",
+        CONST_CONFIGURATION: CONST_CONFIGURATION,
+        CONST_CONDA_AUTH_TYPE: "conda_auth_type",
+    }
+    attribute_map.update(Runtime.attribute_map)
+
+    def with_conda(self, conda_spec: dict = None):
+        if conda_spec.get(self.CONST_CONDA_TYPE) == self.CONST_CONDA_TYPE_SERVICE:
+            raise NotImplementedError(
+                "Service Packs not supported. Please download and re-upload as a custom pack."
+            )
+        elif conda_spec.get(self.CONST_CONDA_TYPE) == self.CONST_CONDA_TYPE_CUSTOM:
+            return self.with_custom_conda(
+                uri=conda_spec.get(self.CONST_CONDA_URI),
+                region=conda_spec.get(self.CONST_CONDA_REGION),
+            )
+        else:
+            raise ValueError(
+                f"Unknown conda type: {conda_spec.get(self.CONST_CONDA_TYPE)}."
+            )
+
+    def with_service_conda(self, slug: str):
+        raise NotImplementedError(
+            "Publish this conda pack first, and provide the published conda pack uri."
+        )
+
+    def with_custom_conda(self, uri: str, region: str = None, auth_type: str = None):
+        """Specifies the custom conda pack for running the job
+
+        Parameters
+        ----------
+        uri : str
+            The OCI object storage URI for the conda pack,
+            e.g. "oci://your_bucket@namespace/object_name."
+            In the Environment Explorer of an OCI notebook session,
+            this is shown as the "source" of the conda pack.
+        region: str, optional
+            The region of the bucket storing the custom conda pack, by default None.
+            If region is not specified, ADS will use the region from your authentication credentials,
+            * For API Key, config["region"] is used.
+            * For Resource Principal, signer.region is used.
+            This is required if the conda pack is stored in a different region.
+        auth_type: str, (="resource_principal")
+            One of "resource_principal", "api_keys", "instance_principal", etc.
+            Auth mechanism used to read the conda back uri provided.
+
+        Returns
+        -------
+        self
+            The runtime instance.
+
+        See Also
+        --------
+        https://docs.oracle.com/en-us/iaas/data-science/using/conda_publishs_object.htm
+
+        """
+        if not auth_type:
+            auth_type = "resource_principal"
+        self.set_spec(self.CONST_CONDA_AUTH_TYPE, auth_type)
+        return super().with_custom_conda(uri=uri, region=region)
 
     def with_archive_uri(self, uri: str) -> "DataFlowRuntime":
         """
@@ -612,6 +703,29 @@ class DataFlowRuntime(Runtime):
         """Bucket to save archive zip"""
         return self.get_spec(self.CONST_ARCHIVE_BUCKET)
 
+    def with_configuration(self, config: dict) -> "DataFlowRuntime":
+        """
+        Set Configuration for Spark.
+
+        Parameters
+        ----------
+        config: dict
+            dictionary of configuration details
+            https://spark.apache.org/docs/latest/configuration.html#available-properties.
+            Example: { “spark.app.name” : “My App Name”, “spark.shuffle.io.maxRetries” : “4” }
+
+        Returns
+        -------
+        DataFlowRuntime
+            runtime instance itself
+        """
+        return self.set_spec(self.CONST_CONFIGURATION, config)
+
+    @property
+    def configuration(self) -> dict:
+        """Configuration for Spark"""
+        return self.get_spec(self.CONST_CONFIGURATION)
+
     def convert(self, **kwargs):
         pass
 
