@@ -37,15 +37,17 @@ Examples
 >>> print(deployment_info.url)
 >>> deployer.list_deployments() # Optionally pass in a status
 """
-from typing import Union, Dict
+from typing import Dict, Union
 
-import pandas as pd
 import oci.pagination
+import pandas as pd
 from ads.common.auth import default_signer
+from oci.data_science.data_science_client import DataScienceClient
+from oci.data_science import DataScienceClientCompositeOperations
+
 from .common import utils
 from .common.utils import OCIClientManager, State
-
-from .model_deployment import ModelDeployment, DEFAULT_WAIT_TIME, DEFAULT_POLL_INTERVAL
+from .model_deployment import DEFAULT_POLL_INTERVAL, DEFAULT_WAIT_TIME, ModelDeployment
 from .model_deployment_properties import ModelDeploymentProperties
 
 
@@ -78,7 +80,7 @@ class ModelDeployer:
         shows the deployments filtered by `status` in a Dataframe
     """
 
-    def __init__(self, config: dict = None):
+    def __init__(self, config: dict = None, ds_client: DataScienceClient = None):
         """Initializes model deployer.
 
         Parameters
@@ -87,13 +89,27 @@ class ModelDeployer:
             ADS auth dictionary for OCI authentication.
             This can be generated by calling ads.common.auth.api_keys() or ads.common.auth.resource_principal().
             If this is None, ads.common.default_signer(client_kwargs) will be used.
+
+         ds_client: oci.data_science.data_science_client.DataScienceClient
+            The Oracle DataScience client.
         """
-        if not config:
-            config = default_signer()
-        self.config = config
-        self.client_manager = OCIClientManager(config)
-        self.ds_client = self.client_manager.ds_client
-        self.ds_composite_client = self.client_manager.ds_composite_client
+        self.ds_client = None
+        self.ds_composite_client = None
+
+        if ds_client:
+            self.ds_client = ds_client
+            self.ds_composite_client = DataScienceClientCompositeOperations(
+                self.ds_client
+            )
+
+        if not (self.ds_client != None and self.ds_composite_client != None):
+            if not config:
+                config = default_signer()
+            self.config = config
+
+            self.client_manager = OCIClientManager(config)
+            self.ds_client = self.client_manager.ds_client
+            self.ds_composite_client = self.client_manager.ds_composite_client
 
     def deploy(
         self,
