@@ -4,6 +4,8 @@
 # Copyright (c) 2022 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
+from enum import Enum
+
 DEFAULT_OCI_CONFIG_FILE = "~/.oci/config"
 DEFAULT_PROFILE = "DEFAULT"
 DEFAULT_CONDA_PACK_FOLDER = "~/conda"
@@ -17,9 +19,17 @@ DEFAULT_MANIFEST_VERSION = "1.0"
 ADS_CONFIG_FILE_NAME = "config.ini"
 ADS_JOBS_CONFIG_FILE_NAME = "ml_job_config.ini"
 ADS_DATAFLOW_CONFIG_FILE_NAME = "dataflow_config.ini"
+ADS_ML_PIPELINE_CONFIG_FILE_NAME = "ml_pipeline.ini"
+ADS_LOCAL_BACKEND_CONFIG_FILE_NAME = "local_backend.ini"
 DEFAULT_IMAGE_HOME_DIR = "/home/datascience"
 DEFAULT_IMAGE_SCRIPT_DIR = "/etc/datascience"
 DEFAULT_IMAGE_CONDA_DIR = "/opt/conda/envs"
 DEFAULT_NOTEBOOK_SESSION_SPARK_CONF_DIR = "/home/datascience/spark_conf_dir"
 DEFAULT_NOTEBOOK_SESSION_CONDA_DIR = "/home/datascience/conda"
-ADS_DATAFLOW_CONFIG_FILE_NAME = "dataflow_config.ini"
+
+
+class BACKEND_NAME(Enum):
+    JOB = "job"
+    DATAFLOW = "dataflow"
+    PIPELINE = "pipeline"
+    LOCAL = "local"
