@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*--
 
-# Copyright (c) 2021, 2022 Oracle and/or its affiliates.
+# Copyright (c) 2021, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 """
@@ -36,7 +36,6 @@ CX_ORACLE = "cx_Oracle"
 PYTHON_ORACLEDB = "PYTHON_ORACLEDB"
 PYTHON_DRIVER_NAME = None
 
-
 try:
     import oracledb as oracle_driver  # Both the driver share same signature for the APIs that we are using.
 
@@ -148,12 +147,17 @@ class OracleRDBMSConnection(oracle_driver.Connection):
             self.tns_entries[name] = entry
 
     def insert(
-        self, table_name: str, df: pd.DataFrame, if_exists: str, batch_size=100000
+        self,
+        table_name: str,
+        df: pd.DataFrame,
+        if_exists: str,
+        batch_size=100000,
+        encoding="utf-8",
     ):
 
         if if_exists not in ["fail", "replace", "append"]:
             raise ValueError(
-                "Unknown option `if_exists`={if_exists}. Valid options are 'fail', 'replace', 'append'"
+                f"Unknown option `if_exists`={if_exists}. Valid options are 'fail', 'replace', 'append'"
             )
         start_time = time()
 
@@ -194,18 +198,14 @@ class OracleRDBMSConnection(oracle_driver.Connection):
                 "float64": "FLOAT",
                 "datetime64": "TIMESTAMP",
             }
-            # add in any string types as Oracle's VARCHAR type setting length to accomodate longest
-            def get_max_str_len(df, column):
-                max_str_len = df[column].dropna().astype(str).str.len().max()
-                if max_str_len >= 4000:
-                    raise Exception(
-                        f'Error: Column "{column}" is too long to store in a VARCHAR2 column'
-                    )
-                return max_str_len
+
+            # add in any string types as Oracle's VARCHAR type setting length to accommodate longest
+            def get_max_str_len(df, column, encoding):
+                return df[column].dropna().str.encode(encoding).map(len).max()
 
             longest_string_column = max(
                 (
-                    get_max_str_len(df_orcl, c)
+                    get_max_str_len(df_orcl, c, encoding)
                     for c in df_orcl.select_dtypes(
                         include=["object", "category"]
                     ).columns
@@ -216,12 +216,11 @@ class OracleRDBMSConnection(oracle_driver.Connection):
             logger.debug(f"Max string column value: {longest_string_column}")
 
             datatypes = {
-                c: f"VARCHAR2({get_max_str_len(df_orcl, c)})"
+                c: f"VARCHAR2({get_max_str_len(df_orcl, c, encoding)})"
                 for c in df_orcl.select_dtypes(include=["object", "category"]).columns
             }
 
             for df_type, orcl_type in type_mappings.items():
-
                 datatypes.update(
                     {
                         column: orcl_type
@@ -247,7 +246,12 @@ class OracleRDBMSConnection(oracle_driver.Connection):
                     + ")"
                 )
                 logger.info(sql)
-                cursor.execute(sql)
+                try:
+                    cursor.execute(sql)
+                except Exception as e:
+                    raise Exception(
+                        f"'{e if e.args else 'Unexpected error encountered'}' with query: '{sql}'"
+                    ) from e
 
             # insert
             bind_columns = ", ".join([f"{col}" for col in df_orcl.columns])
@@ -256,7 +260,7 @@ class OracleRDBMSConnection(oracle_driver.Connection):
 
             logger.info(sql)
 
-            # prevent buffer reallocations by locking in the longest string value
+            # prevent buffer reallocation by locking in the longest string value
             # cursor.setinputsizes(None, longest_string_column)
 
             # replace NaN with None before turning into database records, important - don't
